
#############################################################


P.mat <- function(K)
	{
	# penalty matrix for density
	D <- diag(rep(1,K))
	D <- diff(diff(D))
	P <- t(D)%*%D 
	return(P)
	}


#############################################################


B.basis <- function(x,knots)
	{
	delta <- knots[2]-knots[1]
	n <- length(x)
	K <- length(knots)
	B <- matrix(0,n,K+1)
	for (jj in 1:(K-1))
      	{
		act.inds <- (1:n)[(x>=knots[jj])&(x<=knots[jj+1])]
		act.x <- x[act.inds]
		resc.x <- (act.x-knots[jj])/(knots[jj+1]-knots[jj])
        
		B[act.inds,jj] <- (1/2)*(1-resc.x)^2
		B[act.inds,jj+1] <- -(resc.x^2)+resc.x+1/2
		B[act.inds,jj+2] <- (resc.x^2)/2
		}
	return(B)
	}


#############################################################


rskewnorm <- function(n,mean,sd,skewness)
	{
	c = 2/3.1415926
	delta = skewness/sqrt(1+skewness^2)
	xi = - (sqrt(c) * skewness)/sqrt(1 + skewness^2 * (1-c))
	omega = sqrt(1+xi^2)

	y = delta * abs(rnorm(n,0,1)) + (1-delta^2)^(1/2) * rnorm(n,0,1)
	y = xi + omega * y
	return(mean+sd*y)
	}


dskewnorm <- function(x,mean,sd,skewness)
	{
	c = 2/3.1415926
	delta = skewness/sqrt(1+skewness^2)
	zeta1 = delta * sqrt(c)
	zeta2 = sqrt(1 - c*delta^2)
	y = numeric(length(x))
	xmod = zeta1 + zeta2*(x-mean)/sd
	for(i in 1:length(x))
      	y[i] = (2*zeta2/sd[i]) * dnorm(xmod[i]) * pnorm(skewness*xmod[i])
	return(y)
	}


#############################################################


d.scaled.restricted.mix.norm <- function(x,mean,sd,pi,params)
	{
	if(is.matrix(params))
		{
		p = params[,1]
		mu_curl = params[,2]
		sigmasq1 = params[,3]
		sigmasq2 = params[,4]
		}
	else
		{
		p = params[1]
		mu_curl = params[2]
		sigmasq1 = params[3]
		sigmasq2 = params[4]
		}

	c1 = (1-p)/(p^2+(1-p)^2)
	c2 = -p/(p^2+(1-p)^2)

    
    	mu1 = c1*mu_curl
	mu2 = c2*mu_curl

	sd.e = sqrt(var.e.fn(pi,params))
	y = matrix(0,nrow=length(p),ncol=length(x))
	for(kk in 1:length(p))
	    	y[kk,] = pi[kk]*(p[kk]*dnorm(x,(mean+sd*mu1[kk])/sd.e,sd*sqrt(sigmasq1[kk])/sd.e) + (1-p[kk])*dnorm(x,(mean+sd*mu2[kk])/sd.e,sd*sqrt(sigmasq2[kk])/sd.e))
	y = colSums(y)
	return(y)
	}



d.restricted.mix.norm <- function(x,mean,sd,params)
	{
	if(is.matrix(params))
		{
		p = params[,1]
		mu_curl = params[,2]
		sigmasq1 = params[,3]
		sigmasq2 = params[,4]
		}
	else
		{
		p = params[1]
		mu_curl = params[2]
		sigmasq1 = params[3]
		sigmasq2 = params[4]
		}


	c1 = (1-p)/(p^2+(1-p)^2)
	c2 = -p/(p^2+(1-p)^2)

    
    	mu1 = c1*mu_curl
	mu2 = c2*mu_curl


    	y = p*dnorm(x,mean+sd*mu1,sd*sqrt(sigmasq1)) + (1-p)*dnorm(x,mean+sd*mu2,sd*sqrt(sigmasq2))
	return(y)
	}



mixnorm <- function(n,pi,p,mu_curl,sigmasq1,sigmasq2,e.grid,plot=TRUE)
	{
	m = length(pi)
	y = numeric(n)
	density <- numeric(length(e.grid))
	inds = sample(1:m,n,TRUE,prob=pi)
	for(ii in 1:m)
		{
		temp = which(inds==ii)
		y[temp] = r.restricted.mix.norm(length(temp),c(p[ii],mu_curl[ii],sigmasq1[ii],sigmasq2[ii]))
		}
	params = cbind(p,mu_curl,sigmasq1,sigmasq2)
	y = y/sqrt(var.e.fn(pi,params))
	density <- d.scaled.restricted.mix.norm(e.grid,mean=0,sd=1,pi,params)
	if(plot)
		{
		hist(y,xlim=c(min(e.grid),max(e.grid)),breaks=30,freq=FALSE)
		points(e.grid,density,type="l")
		}

	return(list(es=y,density=density))
	}


r.restricted.mix.norm <- function(n,params)
	{
	p = params[1]
	mu_curl = params[2]
	sigmasq1 = params[3]
	sigmasq2 = params[4]

	c1 = (1-p)/(p^2+(1-p)^2)
	c2 = -p/(p^2+(1-p)^2)

    
    	mu1 = c1*mu_curl
	mu2 = c2*mu_curl


    	inds = sample(0:1,n,TRUE,prob=c(p,(1-p)))
	y = numeric(n)
	temp = which(inds==0)
	y[temp] = rnorm(length(temp),mu1,sqrt(sigmasq1)) 
	temp = which(inds==1)
	y[temp] = rnorm(length(temp),mu2,sqrt(sigmasq2))
	return(y)
	}



var.e.fn <- function(pi,params)
	{
	if(is.matrix(params))
		{
		p = params[,1]
		mu_curl = params[,2]
		sigmasq1 = params[,3]
		sigmasq2 = params[,4]
		}
	else
		{
		p = params[1]
		mu_curl = params[2]
		sigmasq1 = params[3]
		sigmasq2 = params[4]
		}

	c1 = (1-p)/(p^2+(1-p)^2)
	c2 = -p/(p^2+(1-p)^2)


    	mu1 = c1*mu_curl
	mu2 = c2*mu_curl


    	y = p*(mu1^{2}+sigmasq1) + (1-p)*(mu2^{2}+sigmasq2)
	y = sum(pi*y)
	return(y)
	}



gamma.e.fn <- function(pi,params)
	{
	if(is.matrix(params))
		{
		p = params[,1]
		mu_curl = params[,2]
		sigmasq1 = params[,3]
		sigmasq2 = params[,4]
		}
	else
		{
		p = params[1]
		mu_curl = params[2]
		sigmasq1 = params[3]
		sigmasq2 = params[4]
		}

	c1 = (1-p)/(p^2+(1-p)^2)
	c2 = -p/(p^2+(1-p)^2)


    mu1 = c1*mu_curl
	mu2 = c2*mu_curl

    m2 = p*(mu1^{2}+sigmasq1) + (1-p)*(mu2^{2}+sigmasq2)
	m2 = sum(pi*m2)

    m3 = p*(mu1^{3}+3*mu1*sigmasq1) + (1-p)*(mu2^{3}+3*mu2*sigmasq2)
	m3 = sum(pi*m3)

    m4 = p*(mu1^{4}+6*mu1^{2}*sigmasq1+3*sigmasq1^{2}) + (1-p)*(mu2^{3}+6*mu2^{2}*sigmasq2+3*sigmasq2^{2})
	m4 = sum(pi*m4)

	gamma1 = m3/m2^{3/2}
	gamma2 = m4/m2^{2}-3

	return(c(gamma1,gamma2))
	}


r.proposal.params.restricted.mix.norm <- function(p.a,p.b,sigmasq.mu_curl,s11,s12,s21,s22)
	{
	p = rbeta(1,p.a,p.b)

	c1 = (1-p)/(p^2+(1-p)^2)
	c2 = -p/(p^2+(1-p)^2)
    
	mu_curl = rnorm(1,0,sqrt(sigmasq.mu_curl))
	sigmasq1 = 1/rgamma(1,s11,s12)
	sigmasq2 = 1/rgamma(1,s21,s22)

	y = c(p,mu_curl,sigmasq1,sigmasq2)
	return(y)
	}    


r.tnorm.proposal.params.restricted.mix.norm <- function(params)
	{
	current.p <- params[1]
	current.mu_curl <- params[2]
	current.sigmasq1 <- params[3]
	current.sigmasq2 <- params[4]

	p = rtnorm(1,current.p,0.01,lower=0,upper=1)
	c1 = (1-p)/(p^2+(1-p)^2)
	c2 = -p/(p^2+(1-p)^2)
    
	sigmasq1 = rtnorm(1,current.sigmasq1,0.1,lower=0,upper=Inf)
	sigmasq2 = rtnorm(1,current.sigmasq2,0.1,lower=0,upper=Inf)

    mu_curl = rnorm(1,current.mu_curl,0.1)

	y = c(p,mu_curl,sigmasq1,sigmasq2)
	return(y)
	}    




#############################################################

dlaplace <- function(x,mu,b)
	{
	y = exp(-abs(x-mu)/b)/(2*b)
	return(y)
	}



rlaplace <- function(n,mu,b)
	{
	u = runif(n)
	y = mu - b*sign(u-0.5)*log(1-2*abs(u-0.5))
	return(y)
	}



mixlaplace <- function(n,pi,mu,b,e.grid,plot=TRUE)		# Produces mixtures of Laplace
	{
	m = length(pi)
	y = numeric(n)
	density = numeric(length(e.grid))
	inds = sample(1:m,n,TRUE,pi)
      mu.e = sum(pi*mu)
	sd.e = sqrt(sum(pi*(2*b^{2}+mu^{2}))-mu.e^{2})
	for(ii in 1:m)
		{
		temp = which(inds==ii)
		y[temp] = rlaplace(length(temp),mu[ii],b[ii])
		density = density + pi[ii]*dlaplace(e.grid,(mu[ii]-mu.e)/sd.e,b[ii]/sd.e)
		}
	y = (y-mu.e)/sd.e
	if(plot)
		{
		hist(y,xlim=c(min(e.grid),max(e.grid)),freq=FALSE,breaks=30)
		points(e.grid,density,type="l")
		}
	return(list(es=y,density=density))
	}


gamma.e.Laplace.fn <- function(pi,mu,b)
	{
	m1.dash = sum(pi*mu) 
	m2.dash = sum(pi*(2*b^{2}+mu^{2}))
	m3.dash = sum(pi*(mu^{3}+6*b^{2}*mu))
	m4.dash = sum(pi*(mu^{4}+12*b^{2}*mu^{2}+24*b^{4}))

	m2 = m2.dash-m1.dash^{2}
    	m3 = m3.dash - 3*m2.dash*m1.dash + 2*m1.dash^{3}
	m4 = m4.dash - 4*m3.dash*m1.dash + 6*m2.dash*m1.dash^{2} - 3*m1.dash^{4}

    	gamma1 = m3/m2^{3/2}
	gamma2 = m4/m2^{2}-3

	#print(rbind(m1.dash,m2.dash,m3.dash,m4.dash))
	#print(rbind(m2,m3,m4))

	return(c(gamma1,gamma2))
	}



#############################################################


fr <- function(thetas,xs,mis,knots.t,P.t,s2t,us)			# function to be maximized
	{
	vars = B.basis(xs,knots.t)%*%exp(thetas)
	y = - t(thetas) %*% P.t %*% thetas / (2*s2t) - sum(rep(log(vars),times=mis))/2 - sum(us^2/rep(vars,times=mis))/2
	return(-y)
	}


############################################################


gr <- function(thetas,xs,knots.t,P.t,s2t,us)			# gradient function of fr
	{
	vars = B.basis(xs,knots.t)%*%exp(thetas)
	y = - P.t %*% thetas / s2t
	B.basis.components = matrix(0,nrow=K.t+1,ncol=n)
	for(kk in 1:(K.t+1))
		{
   		thetas.new = rep(0,K.t+1)
		thetas.new[kk] = 1
		B.basis.components[kk,] = B.basis(xs,knots.t)%*%thetas.new
		}
	for(kk in 1:(K.t+1))
		for(ii in 1:n)
			for(jj in (sum(mis[1:ii-1])+1):sum(mis[1:ii]))
				y[kk] = y[kk] - (1 - (us[jj]^2)/vars[ii]) * B.basis.components[kk,ii]*exp(thetas[kk])/(2*vars[ii])
	return(-y)
	}


############################################################


prop.sig.thetas.fn <- function(thetas,xs,mis,us,s2t,K.t,P.t,knots.t,n)
	{
	vars = B.basis(xs,knots.t)%*%exp(thetas)
	prop.sig.thetas = matrix(0,nrow=K.t+1,ncol=K.t+1)
	B.basis.components = matrix(0,nrow=K.t+1,ncol=n);
	for(kk in 1:(K.t+1))
		{
   		thetas.new = rep(0,K.t+1)
		thetas.new[kk] = 1
		B.basis.components[kk,] = B.basis(xs,knots.t)%*%thetas.new
		}

	for(kk in 1:(K.t+1))
		{
		for(ll in kk:(K.t+1))
			{
			if(kk==ll)
	  			for(ii in 1:n)
	     				for(jj in (sum(mis[1:(ii-1)])+1):sum(mis[1:ii]))
	        				prop.sig.thetas[kk,ll] = prop.sig.thetas[kk,ll] + ((us[jj]^2)/vars[ii] - 1/2)*(B.basis.components[kk,ii]*B.basis.components[ll,ii])*exp(thetas[kk]+thetas[ll])/(vars[ii]^2) + (1-(us[jj]^2)/vars[ii])*B.basis.components[kk,ii]*exp(thetas[kk])/(2*vars[ii])
			else
	  			for(ii in 1:n)
	     				for(jj in (sum(mis[1:(ii-1)])+1):sum(mis[1:ii]))
	       					prop.sig.thetas[kk,ll] = prop.sig.thetas[kk,ll] + ((us[jj]^2)/vars[ii] - 1/2)*(B.basis.components[kk,ii]*B.basis.components[ll,ii])*exp(thetas[kk]+thetas[ll])/(vars[ii]^2)
			} 
		}
	for(kk in 2:(K.t+1))
   		for(ll in 1:(kk-1))
			prop.sig.thetas[kk,ll] = prop.sig.thetas[ll,kk]
	prop.sig.thetas = prop.sig.thetas + P.t/s2t
	prop.sig.thetas = round(solve(prop.sig.thetas),4)
	
	return(prop.sig.thetas)
	}


#############################################################


pi.fn <- function(x,alpha,beta,xstar,K)
	{
	w <- pnorm(alpha-beta*abs(x-xstar)^2)
	pi <- numeric(K)
	pi[1] <- w[1]
	for(k in 2:(K-1))
		pi[k] <- w[k]*prod(1-w[1:(k-1)])
	pi[K] <- prod(w[1:(K-1)])
	return(pi)
	}



#############################################################






###########################################################################
# Multivariate Laplace Distribution                                       #
###########################################################################

dmvl <- function(x, mu, Sigma, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(nrow(x)==1) x <- t(x)
     if(!is.matrix(mu)) mu <- rep(mu, each=nrow(x))
     if(missing(Sigma)) Sigma <- diag(ncol(x))
     if(!is.matrix(Sigma)) Sigma <- matrix(Sigma)
     if(!is.positive.definite(Sigma)) 
         stop("Matrix Sigma is not positive-definite.")
     k <- nrow(Sigma)
     Omega <- solve(Sigma)
     ss <- x - mu
     z <- rowSums({ss %*% Omega} * ss)
     dens <- log(2/(2*3.141593)^(k/2)) - 0.5*log(det(Sigma)) + (k/4-1/2)*log(2/z)+log(besselK(sqrt(2*z),k/2-1))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvl <- function(n, mu, Sigma)
     {
     mu <- rbind(mu)
     if(missing(Sigma)) Sigma <- diag(ncol(mu))
     if(!is.matrix(Sigma)) Sigma <- matrix(Sigma)
     if(!is.positive.definite(Sigma))
          stop("Matrix Sigma is not positive-definite.")
     k <- ncol(Sigma)
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     e <- matrix(rexp(n, 1), n, k)
     z <- rmvnorm(n, rep(0, k), Sigma)
     x <- mu + sqrt(e)*z
     return(x)
     }



#############################################################







